require("editor_gui")
require("export")
editor={
	shapes={};
	joints={}
}
function editor.load()
	test = false

	screen={}
	screen.width = love.graphics.getWidth()
	screen.height = love.graphics.getHeight()
	
	mode = "selection"
	editor.temp={}
	editor.selected = -1
	editor.camX = -screen.width/2
	editor.camY = -screen.height/2
	editor.oldMX = 0
	editor.oldMY = 0
	editor.grid = true
	editor.gridSizes = {8,16,32,64,128,256}
	editor.gridSize = math.floor(editor.gridSizes[#editor.gridSizes]/2)
	editor.gridX = 0
	editor.gridY = 0
	editor.gridSnap = true
	editor_gui.load()
end

function editor.clearTemp()
	table.destroy(editor.temp)
end

function editor.getGridMousePosition()
	local mx,my = love.mouse.getPosition()
	local offsetX = editor.camX - editor.gridX - math.multiple(editor.camX - editor.gridX,editor.gridSize)
	local offsetY = editor.camY - editor.gridY - math.multiple(editor.camY - editor.gridY,editor.gridSize)
	local dx, dy = math.multiple(mx+offsetX,editor.gridSize) - offsetX, math.multiple(my+offsetY,editor.gridSize) - offsetY
	return dx,dy
end

function editor.toggleGrid()
	if editor.grid == true then editor.grid = false return end
	if editor.grid == false then editor.grid = true return end
end

function editor.changeGridSize(operation)
	local currentSize
	for i=1,#editor.gridSizes do
		if editor.gridSizes[i] == editor.gridSize then currentSize = i end
	end
	if operation == "-" and currentSize > 1 then editor.gridSize = editor.gridSizes[currentSize-1] end
	if operation == "+" and currentSize < #editor.gridSizes then editor.gridSize = editor.gridSizes[currentSize+1] end
end

function editor.toggleGridSnap()
	if editor.gridSnap == true then editor.gridSnap = false return end
	if editor.gridSnap == false then editor.gridSnap = true return end
end

function editor.keypressed(key)
	if key == "t" and not textbox.hasfocus() then
		editor_test.start()
	end
	if key == "delete" and mode == "selection" then
		Shape_destroy(editor.selected)
	end
	editor_gui.keypressed(key)
end

function editor.mousepressed(x,y,button)
	editor_gui.mousepressed(x,y,button)
	if love.mouse.getY() <= 30 then return end
	if editor.gridSnap then x,y = editor.getGridMousePosition() end
	x,y = x + editor.camX, y + editor.camY --offset camara
	if button == "l" then
		if mode == "selection" then
			local mx,my = love.mouse.getPosition()
			mx, my = mx + editor.camX, my + editor.camY
			for i,v in ipairs(editor.shapes) do
				if v.kind == "rectangle" then
					if CheckCollision(mx,my,1,1,v.x,v.y,v.w,v.h) then
						editor.selected = v.id
						break
					end
				elseif v.kind == "polygon" then
					if CheckCollision(mx,my,1,1,polyAABB(v.vertex)) and point_in_polygon(mx,my,v.vertex) then
						editor.selected = v.id
						break
					end
				elseif v.kind == "circle" then
					if distance(mx,my,v.x,v.y) <= v.radius then
						editor.selected = v.id
						break
					end
				elseif v.kind == "edge" then
					if line_vs_circle(v.x1, v.y1, v.x2, v.y2, mx, my, 5) then
						editor.selected = v.id
						break
					end
				elseif v.kind == "chain" then
					local check = false
					for i=1,#v.vertex do
						if not (i % 2 == 0) and not check then
							for ii=1,#v.vertex do
								if ii == i + 1 then
									local x1,y1,x2,y2 = v.vertex[i], v.vertex[ii], v.vertex[ii+1], v.vertex[ii+2]
									if not x2 or not y2 then
										x2 = v.vertex[1]
										y2 = v.vertex[2]
									end
									if line_vs_circle(x1,y1,x2,y2, mx, my, 5) then
										editor.selected = v.id
										check = true
									end
								end
							end
						end
					end
					if check then
						break
					end
				end
				editor.selected = -1
			end
		elseif mode == "rectshape_create" then
			table.insert(editor.temp,x)
			table.insert(editor.temp,y)
		elseif mode == "polshape_create" then
			table.insert(editor.temp,x)
			table.insert(editor.temp,y)
			if #editor.temp >= 6 and distance(editor.temp[1],editor.temp[2],editor.temp[#editor.temp-1],editor.temp[#editor.temp]) < 5 then
				for i = 1,2 do table.remove(editor.temp,#editor.temp) end
			end
			if distance(x,y,editor.temp[1],editor.temp[2]) < 5 and #editor.temp >= 6 or #editor.temp/2 == 8 then
				local vertex={}
				for i=1,#editor.temp do
					table.insert(vertex,editor.temp[i])
				end
				polygonShape_create(vertex)
				editor.clearTemp()
			end
		elseif mode == "circshape_create" then
			table.insert(editor.temp,x)
			table.insert(editor.temp,y)
		elseif mode == "edgeshape_create" then
			table.insert(editor.temp,x)
			table.insert(editor.temp,y)
			if #editor.temp >= 4 then
				edgeShape_create(editor.temp[1],editor.temp[2],editor.temp[3],editor.temp[4])
				editor.clearTemp()
			end
		elseif mode == "chainshape_create" then --Como el polygon, pero sin limites
			table.insert(editor.temp,x)
			table.insert(editor.temp,y)
		end
	elseif button == "r" then
		if mode == "polshape_create" then
			if #editor.temp >= 4 then
				if distance(x,y,editor.temp[1],editor.temp[2]) > 5 then
					table.insert(editor.temp,x)
					table.insert(editor.temp,y)
				else
					return
				end
			else
				return
			end
			local vertex={}
			for i=1,#editor.temp do
				table.insert(vertex,editor.temp[i])
			end
			polygonShape_create(vertex)
			editor.clearTemp()
		elseif mode == "chainshape_create" then
			if #editor.temp >= 4 then
				table.insert(editor.temp,x)
				table.insert(editor.temp,y)
				local vertex={}
				for i=1,#editor.temp do
					table.insert(vertex,editor.temp[i])
				end
				chainShape_create(vertex)
				editor.clearTemp()
			end
		end
	end
end

function rectangleShape_create(x,y,w,h)
	if w > -1 and w < 1 or h > -1 and h < 1 then return end
	if math.sign(w) == -1 then
		x = x + w
		w = -w
	end
	if math.sign(h) == -1 then
		y = y + h
		h = -h
	end
	local shape={}
	shape.x = x
	shape.y = y
	shape.w = w
	shape.h = h
	shape.kind = "rectangle"
	
	shape.id = getID(editor.shapes)
	table.insert(editor.shapes,shape)
end

function polygonShape_create(vertex)
	local shape={}
	shape.vertex={}
	for i=1,#vertex do
		table.insert(shape.vertex,vertex[i])
	end
	shape.kind = "polygon"
	
	shape.id = getID(editor.shapes)
	table.insert(editor.shapes,shape)
end

function circleShape_create(x,y,radius)
	if radius <= 5 then return end
	local shape={}
	shape.x = x
	shape.y = y
	shape.kind = "circle"
	
	shape.radius = radius
	shape.id = getID(editor.shapes)
	table.insert(editor.shapes,shape)
end

function edgeShape_create(x1,y1,x2,y2)
	local shape={}
	shape.x1 = x1
	shape.y1 = y1
	shape.x2 = x2
	shape.y2 = y2
	shape.kind = "edge"
	
	shape.id = getID(editor.shapes)
	table.insert(editor.shapes,shape)
end

function chainShape_create(vertex)
	local shape={}
	shape.vertex={}
	for i=1,#editor.temp do
		table.insert(shape.vertex,vertex[i])
	end
	shape.kind = "chain"
	
	shape.id = getID(editor.shapes)
	table.insert(editor.shapes,shape)
end

function Shape_destroy(id)
	for i,v in ipairs(editor.shapes) do
		if v.id == id then
			table.remove(editor.shapes,i)
			editor.selected = -1
		end
	end
end

function editor.mousereleased(x,y,button)
	editor_gui.mousereleased(x,y,button)
	if love.mouse.getY() <= 30 then return end
	if editor.gridSnap then x,y = editor.getGridMousePosition() end
	x,y = x + editor.camX, y + editor.camY --offset camara
	if love.mouse.getY() <= 30 then return end
	if button == "l" then
		if mode == "rectshape_create" then
			if #editor.temp >= 2 then
				rectangleShape_create(editor.temp[1],editor.temp[2],x-editor.temp[1],y-editor.temp[2])
				editor.clearTemp()
			end
		elseif mode == "circshape_create" then
			if #editor.temp >= 2 then
				circleShape_create(editor.temp[1],editor.temp[2],distance(editor.temp[1],editor.temp[2],x,y))
				editor.clearTemp()
			end
		end
	end
end

function editor.update(dt)
	--0.9 resizable support
	screen.width = love.graphics.getWidth()
	screen.height = love.graphics.getHeight()
	--arrastre camara
	if love.mouse.isDown("m") then
		editor.camX = editor.camX + (editor.oldMX - love.mouse.getX())
		editor.camY = editor.camY + (editor.oldMY - love.mouse.getY())
	end
	--grid
	local a = math.multipleDown(screen.width,editor.gridSizes[#editor.gridSizes])
	local b = math.multipleDown(screen.height,editor.gridSizes[#editor.gridSizes])
	if editor.camX < editor.gridX then editor.gridX = editor.gridX - a end
	if editor.camY < editor.gridY then editor.gridY = editor.gridY - b end
	if editor.camX > editor.gridX + a then editor.gridX = editor.gridX + a end
	if editor.camY > editor.gridY + b then editor.gridY = editor.gridY + b end
	editor_gui.update(dt)
	editor.oldMX, editor.oldMY = love.mouse.getPosition()
end

function editor.draw()
	local mx,my
	if editor.gridSnap then 
		mx,my = editor.getGridMousePosition()
	else
		mx,my = love.mouse.getPosition()
	end
	mx,my = mx + editor.camX, my + editor.camY --offset camara
	love.graphics.push()
	love.graphics.translate(-editor.camX,-editor.camY)
	--grid
	if editor.grid then
		love.graphics.setColor(255,255,255,20)
		for i=1,math.ceil(screen.width*2/editor.gridSize) do
			love.graphics.line(i*editor.gridSize-editor.gridSize+editor.gridX, editor.gridY, i*editor.gridSize-editor.gridSize+editor.gridX, editor.gridY+screen.height*2)
		end
		for i=1,math.ceil(screen.height*2/editor.gridSize) do
			love.graphics.line(editor.gridX, i*editor.gridSize-editor.gridSize+editor.gridY, screen.width*2+editor.gridX, i*editor.gridSize-editor.gridSize+editor.gridY)
		end
	end
	--lineas azules
	love.graphics.setColor(12,183,242,150)
	love.graphics.line(0,editor.camY,0,editor.camY+screen.height)
	love.graphics.line(editor.camX,0,editor.camX+screen.width,0)
	--shapes
	--Dibujar shapes ya creadas
	for i,v in ipairs(editor.shapes) do
		if v.id == editor.selected then
			love.graphics.setLineStyle("smooth" )
			love.graphics.setColor(204,255,0,255)
		else
			love.graphics.setLineStyle("rough" )
			love.graphics.setColor(50,255,50,255)
		end
		if v.kind == "rectangle" then
			love.graphics.rectangle("line",v.x,v.y,v.w,v.h)
		elseif v.kind == "polygon" then
			love.graphics.polygon("line",v.vertex)
		elseif v.kind == "circle" then
			love.graphics.circle("line",v.x,v.y,v.radius)
		elseif v.kind == "edge" then
			love.graphics.line(v.x1,v.y1,v.x2,v.y2)
		elseif v.kind == "chain" then
			love.graphics.line(v.vertex)
		end
	end
	love.graphics.setLineWidth( 1 )
	love.graphics.setLineStyle("rough" )
	love.graphics.setColor(50,255,50,255)
	love.graphics.setColor(255,0,120,255)
	--Previalisacisodns de shapes en creacion
	if mode == "rectshape_create" then
		if #editor.temp >= 2 then
			love.graphics.rectangle("line",editor.temp[1],editor.temp[2],mx-editor.temp[1],my-editor.temp[2])
		end
	elseif mode == "polshape_create" then
		for i=1,#editor.temp do --Primer loop, encontramos X
			if not (i % 2 == 0) then --Al encontrar X (i = par), pasamos al segundo loop, en busca de Y
				for ii=1,#editor.temp do
					if ii == i + 1 then --Al encontrar Y (i+1), pasamos a dibujar el circulo
						love.graphics.circle("line",editor.temp[i],editor.temp[ii],5)
					end	
				end
			end
		end
		if #editor.temp >= 4 then
			love.graphics.line(editor.temp)
		end
		if #editor.temp >= 2 then
			love.graphics.line(editor.temp[#editor.temp-1],editor.temp[#editor.temp],mx,my)
		end
	elseif mode == "circshape_create" then
		if editor.temp[1] then
			love.graphics.line(editor.temp[1],editor.temp[2],mx,my)
			love.graphics.circle("line",editor.temp[1],editor.temp[2],distance(editor.temp[1],editor.temp[2],mx,my))
		end
	elseif mode == "edgeshape_create" then
		if #editor.temp >= 2 then
			love.graphics.circle("line",editor.temp[1],editor.temp[2],5)
			love.graphics.line(editor.temp[1],editor.temp[2],mx,my)
		end
	elseif mode == "chainshape_create" then
		for i=1,#editor.temp do
			if not (i % 2 == 0) then
				for ii=1,#editor.temp do
					if ii == i + 1 then
						love.graphics.circle("line",editor.temp[i],editor.temp[ii],5)
					end	
				end
			end
		end
		if #editor.temp >= 4 then
			love.graphics.line(editor.temp)
		end
		if #editor.temp >= 2 then
			love.graphics.line(editor.temp[#editor.temp-1],editor.temp[#editor.temp],mx,my)
		end
	end
	love.graphics.setLineStyle("rough")
	love.graphics.setColor(255,255,255,255)
	love.graphics.pop()
	editor_gui.draw()
	--debug
	for i,v in ipairs(editor.temp) do
		love.graphics.print(i..": "..v,25,40+i*10-10)
	end
	love.graphics.print("FPS: "..love.timer.getFPS(),0,screen.height - 12)
end