editor_test={}
objects = {}
function editor_test.load()
	love.physics.setMeter(64)
	mJoint = false
end

function editor_test.start()
	test = true
	safeconsole = {}
	world = love.physics.newWorld(0, 9.81*64, true)
	for _,shape in ipairs(editor.shapes) do
		local object = {}
		object.kind = shape.kind
		if shape.kind == "rectangle" then
			object.body = love.physics.newBody(world, shape.x+shape.w/2, shape.y+shape.h/2,"dynamic")
			object.shape = love.physics.newRectangleShape(shape.w, shape.h)
		elseif shape.kind == "polygon" then
			object.body = love.physics.newBody(world, 0, 0, "dynamic")
			object.shape = love.physics.newPolygonShape(unpack(shape.vertex))
		elseif shape.kind == "circle" then
			object.body = love.physics.newBody(world, shape.x, shape.y, "dynamic")
			object.shape = love.physics.newCircleShape(shape.radius)
		elseif shape.kind == "edge" then
			object.body = love.physics.newBody(world, 0, 0)
			object.shape = love.physics.newEdgeShape(shape.x1,shape.y1,shape.x2,shape.y2)
		elseif shape.kind == "chain" then
			object.body = love.physics.newBody(world, 0, 0)
			object.shape = love.physics.newChainShape(false,unpack(shape.vertex))
			object.vertex = shape.vertex --prevencion del bug +8 vertices
		end
		object.fixture = love.physics.newFixture(object.body,object.shape)
		object.fixture:setRestitution(0.5)
		table.insert(objects,object)
	end
end

function editor_test.stop()
	test = false
	safeconsole = nil
	world:destroy()
	table.destroy(objects)
end

function safeprint(text)
	table.insert(safeconsole,text)
end

function editor_test.keypressed(key)
	if key == "t" then
		editor_test.stop()
	end
end

function editor_test.mousepressed(x,y,button)
	if button == "l" then
		for i, v in ipairs(objects) do
			if v.fixture:testPoint(x + editor.camX, y + editor.camY) then
				mJoint = love.physics.newMouseJoint(v.body, x + editor.camX, y + editor.camY)
				return
			end
		end
	end
end

function editor_test.mousereleased(x,y,button)
	if button == "l" then
		if mJoint then
			mJoint:destroy()
		end
		mJoint = false
	end
end

function editor_test.update(dt)
	if love.mouse.isDown("m") then
		editor.camX = editor.camX + (editor.oldMX - love.mouse.getX())
		editor.camY = editor.camY + (editor.oldMY - love.mouse.getY())
	end
	editor.oldMX, editor.oldMY = love.mouse.getPosition()
	if mJoint then
		local mx = love.mouse.getX() + editor.camX
		local my = love.mouse.getY() + editor.camY
		mJoint:setTarget(mx, my)
	end
	world:update(dt)
end

function editor_test.draw()
	love.graphics.push()
	love.graphics.translate(-editor.camX,-editor.camY)
	for i,v in ipairs(objects) do
		if v.kind == "rectangle" or v.kind == "polygon" then
			love.graphics.setColor(100,100,120,120)
			love.graphics.polygon("fill", v.body:getWorldPoints(v.shape:getPoints()))
			love.graphics.setColor(180,180,220,255)
			love.graphics.polygon("line", v.body:getWorldPoints(v.shape:getPoints()))
		elseif v.kind == "circle" then
			love.graphics.setColor(100,100,120,120)
			love.graphics.circle("fill",v.body:getX(), v.body:getY(),v.shape:getRadius())
			love.graphics.setColor(180,180,220,255)
			love.graphics.circle("line",v.body:getX(), v.body:getY(),v.shape:getRadius())
			local newX, newY = getPointFromDistanceRotation(v.body:getX(), v.body:getY(), v.shape:getRadius(), v.body:getAngle())
			love.graphics.line(v.body:getX(), v.body:getY(), newX, newY)
		elseif v.kind == "edge" then
			love.graphics.setColor(180,180,220,255)
			love.graphics.line(v.shape:getPoints())
		elseif v.kind == "chain" then
			love.graphics.setColor(220,220,220,255)
			love.graphics.line(v.vertex)
		end
	end
	love.graphics.pop()
	--consola
	for i,v in ipairs(safeconsole) do
		love.graphics.print(v,10,i*12-2)
	end
end