export={}
function export.now()
	if not love.filesystem.exists("exports") then
		love.filesystem.createDirectory("exports")
	end
	local filecount = 0
	for i,file in pairs(love.filesystem.getDirectoryItems("exports")) do
		filecount = filecount + 1
	end
	print(filecount)
	local newfilename = "export"
	if filecount > 0 then
		newfilename = newfilename .. "_" .. filecount + 1
	end
	print(newfilename)
	
--string setup
local fcont = [[
function love.load()
	love.physics.setMeter(64)
	world = love.physics.newWorld(0, 9.81*64, true)
	shapes={
]]

	for _,t in ipairs(editor.shapes) do
		fcont=fcont.."		{\n"
		for k,v in pairsByKeys(t) do
			if type(v) == "table" then
				fcont=fcont.."			vertex={\n"
				for i,vrtx in ipairs(v) do
					if not (i % 2 == 0) then
						fcont=fcont.."				"..vrtx..", "
					else
						fcont=fcont.." "..vrtx..",\n"
					end
				end
				fcont=fcont.."			},\n"
			else
				if k == "kind" then
					fcont=fcont.."			"..k..' = "'..v..'",\n'
				else
					fcont=fcont.."			"..k.." = "..v..",\n"
				end
			end
		end
		fcont=fcont..'		},\n'
	end

fcont = fcont .. "	}\n	objects = shapesToWorld(shapes)\nend\n"

fcont = fcont .. "\nfunction shapesToWorld(shapes)\n"

fcont = fcont .. [[
	local objects = {}
	for _,shape in ipairs(shapes) do
		local object = {}
		object.kind = shape.kind
		object.id = shape.id
		if shape.kind == "rectangle" then
			object.body = love.physics.newBody(world, shape.x+shape.w/2, shape.y+shape.h/2,"dynamic")
			object.shape = love.physics.newRectangleShape(shape.w, shape.h)
		elseif shape.kind == "polygon" then
			object.body = love.physics.newBody(world, 0, 0, "dynamic")
			object.shape = love.physics.newPolygonShape(unpack(shape.vertex))
		elseif shape.kind == "circle" then
			object.body = love.physics.newBody(world, shape.x, shape.y, "dynamic")
			object.shape = love.physics.newCircleShape(shape.radius)
		elseif shape.kind == "edge" then
			object.body = love.physics.newBody(world, 0, 0)
			object.shape = love.physics.newEdgeShape(shape.x1,shape.y1,shape.x2,shape.y2)
		elseif shape.kind == "chain" then
			object.body = love.physics.newBody(world, 0, 0)
			object.shape = love.physics.newChainShape(false,unpack(shape.vertex))
			object.vertex = shape.vertex
		end
		object.fixture = love.physics.newFixture(object.body,object.shape)
		table.insert(objects,object)
	end
	return objects
end

function love.update(dt)
	world:update(dt)
end

function love.draw()
	for i,v in ipairs(objects) do
		if v.kind == "rectangle" or v.kind == "polygon" then
			love.graphics.setColor(100,100,120,120)
			love.graphics.polygon("fill", v.body:getWorldPoints(v.shape:getPoints()))
			love.graphics.setColor(180,180,220,255)
			love.graphics.polygon("line", v.body:getWorldPoints(v.shape:getPoints()))
			if v.kind == "rectangle" then
				love.graphics.print(v.id, v.body:getX(), v.body:getY())
			else
				local sx, sy = v.body:getWorldPoints(v.shape:getPoints())
				love.graphics.print(v.id, sx, sy)
			end
		elseif v.kind == "circle" then
			love.graphics.setColor(100,100,120,120)
			love.graphics.circle("fill",v.body:getX(), v.body:getY(),v.shape:getRadius())
			love.graphics.setColor(180,180,220,255)
			love.graphics.circle("line",v.body:getX(), v.body:getY(),v.shape:getRadius())
			love.graphics.print(v.id, v.body:getX(), v.body:getY())
		elseif v.kind == "edge" then
			love.graphics.setColor(180,180,220,255)
			love.graphics.line(v.shape:getPoints())
			local x, y = v.body:getWorldPoints(v.shape:getPoints())
			love.graphics.print(v.id, x, y)
		elseif v.kind == "chain" then
			love.graphics.setColor(220,220,220,255)
			love.graphics.line(v.vertex)
			love.graphics.print(v.id, v.vertex[1], v.vertex[2])
		end
	end
end
]]



love.filesystem.write("exports/"..newfilename..".lua", fcont)
	
end