--Funciones extra desde otros lugares

--foros love2d, distancia entre 2 puntos
function distance ( x1, y1, x2, y2 )
  local dx = x1 - x2
  local dy = y1 - y2
  return math.sqrt ( dx * dx + dy * dy )
end

--alguna parte de stackoverflow, borrar contenido de tabla
function table.destroy(t)
	for k in pairs (t) do
		t [k] = nil
	end
end

--lua-users, ipairs inverso
function ripairs(t)
  local max = 1
  while t[max] ~= nil do
    max = max + 1
  end
  local function ripairs_it(t, i)
    i = i-1
    local v = t[i]
    if v ~= nil then
      return i,v
    else
      return nil
    end
  end
  return ripairs_it, t, max
end

--love2d wiki, AABB
function CheckCollision(ax1,ay1,aw,ah, bx1,by1,bw,bh)
  local ax2,ay2,bx2,by2 = ax1 + aw, ay1 + ah, bx1 + bw, by1 + bh
  return ax1 < bx2 and ax2 > bx1 and ay1 < by2 and ay2 > by1
end

--love2dwiki, math

function math.multiple(n, size) size = size or 10 return math.round(n/size)*size end

function math.multipleDown(n, size) size = size or 10 return math.floor(n/size)*size end

function math.round(n, deci) deci = 10^(deci or 0) return math.floor(n*deci+.5)/deci end

function math.sign(n) return n>0 and 1 or n<0 and -1 or 0 end

--multi theft auto wiki
function getPointFromDistanceRotation(x, y, dist, angle)
 
    local a = angle;
 
    local dx = math.cos(a) * dist;
    local dy = math.sin(a) * dist;
 
    return x+dx, y+dy;
 
end

-- Checks if two line segments intersect. Line segments are given in form of ({x,y},{x,y}, {x,y},{x,y}), love2dwiki
function checkIntersect(l1p1, l1p2, l2p1, l2p2)
    local function checkDir(pt1, pt2, pt3) return math.sign(((pt2.x-pt1.x)*(pt3.y-pt1.y)) - ((pt3.x-pt1.x)*(pt2.y-pt1.y))) end
    return (checkDir(l1p1,l1p2,l2p1) ~= checkDir(l1p1,l1p2,l2p2)) and (checkDir(l2p1,l2p2,l1p1) ~= checkDir(l2p1,l2p2,l1p2))
end

--yo

function point_in_polygon(x,y,vertex)
	local px,py,pw,ph = polyAABB(vertex)
	local check = 1
	for i=1,#vertex do
		if not (i % 2 == 0) then
			for ii=1,#vertex do
				if ii == i + 1 then
					local x1 = vertex[i]
					local y1 = vertex[ii]
					local x2 = vertex[ii+1]
					local y2 = vertex[ii+2]
					if not x2 or not y2 then
						x2 = vertex[1]
						y2 = vertex[2]
					end
					--                    cambiar 'x=x+pw' por algo mejor
					if checkIntersect( {x=x,y=y}, {x=x+pw,y=y}, {x=x1,y=y1}, {x=x2,y=y2} ) then check = -check end
				end
			end
		end
	end
	if check == 1 then return false end
	if check == -1 then return true end
end

function polyAABB(vertex)
	local minX, minY, maxX, maxY = vertex[1],vertex[2],vertex[3],vertex[4]
	for i=1,#vertex do
		if not (i % 2 == 0) then
			for ii=1,#vertex do
				if ii == i + 1 then
					local x,y = vertex[i], vertex[ii]
					if x < minX then minX = x end
					if x > maxX then maxX = x end
					if y < minY then minY = y end
					if y > maxY then maxY = y end
				end
			end
		end
	end
	return minX, minY, maxX-minX, maxY-minY
end

--cutils, bitbucket
-- (a)
--   \ .-'''-.
--   (q)      \
--   | \ (c)   |
--    \ \     /
--     `-...-'
--        \   
--        (b)  

-- Tests segment (a-b) against circle (c)
-- Returns intersection value |a-q| and first point of intersection (q)
function line_vs_circle(ax, ay, bx, by, cx, cy, radius)
  local dx, dy = bx - ax, by - ay
  local d = length(dx, dy)
  dx, dy = dx/d, dy/d

  local mx, my = ax - cx, ay - cy

  local b = dot(mx, my, dx, dy)
  local c = dot(mx, my, mx, my) - radius*radius
  if c > 0 and b > 0 then
    return
  end

  local discr = b*b - c
  if discr < 0 then
    return
  end
  local t = -b - math.sqrt(discr)
  if t < 0 then
    t = 0
  end

  if t > d then
    return
  end

  local qx = ax + t*dx
  local qy = ay + t*dy

  return t, qx, qy
end

-- Returns the dot product of two points (a,b)
function dot(ax, ay, bx, by)
  return ax*bx + ay*by
end

-- Returns the length of a vector
function length(x, y)
  return math.sqrt(x*x + y*y)
end

-- Original author: Robin
-- https://love2d.org/forums/viewtopic.php?f=5&t=1323
function love.graphics.roundrect(mode, x, y, width, height, xround, yround)
	local points = {}
	local precision = (xround + yround) * .1
	local tI, hP = table.insert, .5*math.pi
	if xround > width*.5 then xround = width*.5 end
	if yround > height*.5 then yround = height*.5 end
	local X1, Y1, X2, Y2 = x + xround, y + yround, x + width - xround, y + height - yround
	local sin, cos = math.sin, math.cos
	for i = 0, precision do
		local a = (i/precision-1)*hP
		tI(points, X2 + xround*cos(a))
		tI(points, Y1 + yround*sin(a))
	end
	for i = 0, precision do
		local a = (i/precision)*hP
		tI(points, X2 + xround*cos(a))
		tI(points, Y2 + yround*sin(a))
	end
	for i = 0, precision do
		local a = (i/precision+1)*hP
		tI(points, X1 + xround*cos(a))
		tI(points, Y2 + yround*sin(a))
	end
	for i = 0, precision do
		local a = (i/precision+2)*hP
		tI(points, X1 + xround*cos(a))
		tI(points, Y1 + yround*sin(a))
	end
	love.graphics.polygon(mode, unpack(points))
end

--yo
function getID(t)
	if t[1] then
		return t[#t].id + 1
	else
		return 1
	end
end

function isboolean(text)
	if text == "true" or text == "false" then
		return true
	else
		return false
	end
end

--por ai
function toboolean(v)
    return (type(v) == "string" and v == "true") or (type(v) == "number" and v ~= 0) or (type(v) == "boolean" and v)
end

--http://www.lua.org/pil/19.3.html
function pairsByKeys (t, f)
      local a = {}
      for n in pairs(t) do table.insert(a, n) end
      table.sort(a, f)
      local i = 0      -- iterator variable
      local iter = function ()   -- iterator function
        i = i + 1
        if a[i] == nil then return nil
        else return a[i], t[a[i]]
        end
      end
      return iter
end