textboxes={}
textbox={}

function textbox.load()
	textbox.timer = 0
	textbox.maxTimer = 1
	textbox.showBar = false
end

function textbox.create(x,y,w,h)
	local t={}
	t.x = x
	t.y = y
	t.w = w
	t.h = h
	t.text = ""
	t.rtext = ""
	t.focus = false
	t.limit = "letters&numbers"
	t.id = getID(textboxes)
	table.insert(textboxes,t)
	return t.id
end

function textbox.destroy(id)
	for i,v in ipairs(textboxes) do
		if v.id == id then
			table.remove(textboxes,i)
		end
	end
end

function textbox.setText(id,text)
	for i,v in ipairs(textboxes) do
		if v.id == id then
			v.text = text
			v.rtext = text
		end
	end
end

function textbox.getText()
	for i,v in ipairs(textboxes) do
		if v.id == id then
			return v.rtext
		end
	end
end

function textbox.hasfocus()
	for i,v in ipairs(textboxes) do
		if v.focus then return true end
	end
	return false
end

function textbox.keypressed(key)
	for i,v in ipairs(textboxes) do
		if v.focus then
			if key == "backspace" then
				v.text = string.sub(v.text,0,string.len(v.text)-1)
			elseif string.find(key,"kp") then
				if font:getWidth(v.text..string.sub(key,3)) >= v.w-5 then break end
				v.text = v.text .. string.sub(key,3)
			elseif string.len(key) == 1 then
				if font:getWidth(v.text..key) >= v.w-5 then break end
				v.text = v.text .. key
			elseif key == "return" then
				v.rtext = v.text
				for i,v in ipairs(textboxes) do
					v.focus = false
				end
				return v.id, v.rtext
			end		
		end
	end
end

function textbox.mousepressed(x,y,button)
	for i,v in ipairs(textboxes) do
		if CheckCollision(v.x,v.y,v.w,v.h,x,y,1,1) then
			v.focus = true
			textbox.timer = 0
			textbox.showBar = true
		else
			v.focus = false
		end
	end
end

function textbox.setPosition(id,x,y)
	for i,v in ipairs(textboxes) do
		if v.id == id then
			v.x = x
			v.y = y
		end
	end
end

function textbox.getPosition(id)
	for i,v in ipairs(textboxes) do
		if v.id == id then
			return v.x,v.y
		end
	end
end

function textbox.update(dt)
	textbox.timer = textbox.timer + dt
	if textbox.timer >= textbox.maxTimer then
		textbox.showBar = not textbox.showBar
		textbox.timer = 0
	end
end

function textbox.draw()
	for i,v in ipairs(textboxes) do
		love.graphics.setColor(255,255,255,255)
		love.graphics.rectangle("fill",v.x,v.y,v.w,v.h)
		if v.rtext ~= v.text then
			love.graphics.setColor(255,0,0,100)
			love.graphics.rectangle("fill",v.x,v.y,v.w,v.h)
		end
		love.graphics.setColor(0,0,0,255)
		love.graphics.rectangle("line",v.x,v.y,v.w,v.h)
		love.graphics.print(v.text,v.x+2,v.y+2)
		if v.focus and textbox.showBar then
			love.graphics.line(v.x+font:getWidth(v.text)+4,v.y+2,v.x+font:getWidth(v.text)+4,v.y+v.h-1)
		end
	end
end